@@ -293,7 +293,7 @@ main(int argc, char **argv)
   char *rootdir = 0;
   char *keyname = 0;
   int keyname_depstr = 0;
-  int keyname_alldeps = 0;		/* dnf repoquesy --alldeps */
+  int keyname_alldeps = 0;		/* dnf repoquery --alldeps */
   int debuglevel = 0;
   int answer, acnt = 0;
   char *testcase = 0;
@@ -385,7 +385,7 @@ main(int argc, char **argv)
 	}
       else if (argc > 1 && !strcmp(argv[1], "--alldeps"))
 	{
-	  keyname_alldeps = 1;		/* dnf repoquesy --alldeps */
+	  keyname_alldeps = 1;		/* dnf repoquery --alldeps */
 	  argc--;
 	  argv++;
 	}
@@ -788,14 +788,17 @@ rerunsolver:
       for (problem = 1; problem <= pcnt; problem++)
 	{
 	  int take = 0;
-	  printf("Problem %d/%d:\n", problem, pcnt);
-	  solver_printprobleminfo(solv, problem);
-	  printf("\n");
+	  printf("Problem %d/%d:\n%s\n\n", problem, pcnt, solver_problem2str(solv, problem));
 	  scnt = solver_solution_count(solv, problem);
 	  for (solution = 1; solution <= scnt; solution++)
 	    {
+	      Queue sq;
 	      printf("Solution %d:\n", solution);
-	      solver_printsolution(solv, problem, solution);
+	      queue_init(&sq);
+	      solver_all_solutionelements(solv, problem, solution, 1, &sq);
+	      for (i = 0; i < sq.count; i += 3)
+		printf("  - %s\n", solver_solutionelementtype2str(solv, sq.elements[i], sq.elements[i + 1], sq.elements[i + 2]));
+	      queue_free(&sq);
 	      printf("\n");
 	    }
 	  for (;;)
